from Aggregates.message_aggregates import *
from . import enums

def calculateLongitude(decodedValue):
    degrees = decodedValue/60
    if degrees == 181:
        return "not available"
    elif degrees > 0:
        return str(abs(degrees)) + "degrees (East)"
    elif degrees < 0:
        return str(abs(degrees)) + "degrees (West)"
    else:
        return "0 degrees"

def calculateLatitude(decodedValue):
    degrees = decodedValue/60
    if degrees == 91:
        return "not available"
    elif degrees > 0:
        return str(abs(degrees)) + "degrees (North)"
    elif degrees < 0:
        return str(abs(degrees)) + "degrees (South)"
    else:
        return "0 degrees"

def interpretAccuracy(decodedValue):
    acc_values = ["unaugmented GNSS fix with accuracy > 10m", "DGPS-quality fix with an accuracy of < 10ms"]
    return acc_values[decodedValue]

def interpretRAIMflag(decodedValue):
    if decodedValue == 0:
        return "RAIM not in use"
    else:
        return "RAIM in use"

def interpretRetransmitFlag(decodedValue):
    retransmit = ["no retransmit", "retransmitted"]
    return retransmit[decodedValue]

def calculateSpeedOverGround(decodedValue):
    if decodedValue == 102.2:
        return "102.2 knots or higher"
    elif decodedValue == 102.3:
        return "not available"
    else:
        return str(decodedValue) + " knots"

def interpretPersonsOnBoard(decodedValue):
    if  decodedValue == 0:
        return "N/A"
    elif decodedValue == 8191:
        return "8191 or more"
    else:
        return decodedValue   

def concatenateFields(dataArray, mainFields):
    for element in dataArray:
        mainFields = mainFields + element.type
        mainFields = mainFields + element.fields

def concatenateDataparts(dataArray, mainDataparts):
    for element in dataArray:
        mainDataparts = mainDataparts.append("")
        mainDataparts = mainDataparts + element.dataParts

def updateType_1(message):
    to_update_index = message.fields.index("Navigation Status")
    message.dataParts[to_update_index] = enums.navigationStatus[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Rate of Turn")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "not turning"
    elif message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "turning right at more than 5deg/30s (No TI available)"
    elif message.dataParts[to_update_index] == -127:
        message.dataParts[to_update_index] = "turning left at more than 5deg/30s (No TI available)"
    elif message.dataParts[to_update_index] == 128:
        message.dataParts[to_update_index] = "no turn information available (default)"
    elif message.dataParts[to_update_index] < 0:
        div = message.dataParts[to_update_index]/4.773
        message.dataParts[to_update_index] = " turning left at {} degrees/min".format(div * div)
    elif message.dataParts[to_update_index] > 0:
        div = message.dataParts[to_update_index]/4.773
        message.dataParts[to_update_index] = " turning right at " + str(div * div) + " degrees/min"

    to_update_index = message.fields.index("Speed Over Ground")
    message.dataParts[to_update_index] = calculateSpeedOverGround(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Position Accuracy")
    message.dataParts[to_update_index] = interpretAccuracy(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Course Over Ground")
    if message.dataParts[to_update_index] == 360.0:
        message.dataParts[to_update_index] = "not available"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + " degrees"

    to_update_index = message.fields.index("True Heading")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "not available"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + " degrees"

    to_update_index = message.fields.index("Time Stamp")
    if message.dataParts[to_update_index] == 60:
        message.dataParts[to_update_index] = "not available"
    elif message.dataParts[to_update_index] == 61:
        message.dataParts[to_update_index] = "positioning system in manual input mode"
    elif message.dataParts[to_update_index] == 62:
        message.dataParts[to_update_index] == "Electronic Position Fixing System operates in estimated (dead reckoning) mode"
    elif message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] == "positioning system is inoperative"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + "second of UTC timestamp"

    to_update_index = message.fields.index("Maneuver Indicator")
    message.dataParts[to_update_index] = enums.maneuverIndicator[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("RAIM flag")
    message.dataParts[to_update_index] = interpretRAIMflag(message.dataParts[to_update_index])

def updateType_2(message):
    updateType_1(message)

def updateType_3(message):
    updateType_1(message)

def updateType_4(message):
    to_update_index = message.fields.index("Year (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Month (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Minute (UTC)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Second (UTC)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Fix quality")
    message.dataParts[to_update_index] = interpretAccuracy(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Longitude")
    message.dataParts[to_update_index] = calculateLongitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Latitude")
    message.dataParts[to_update_index] = calculateLatitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Type of EPFD")
    message.dataParts[to_update_index] = enums.ePFDFixTypes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("RAIM flag")
    message.dataParts[to_update_index] = interpretRAIMflag(message.dataParts[to_update_index])

def updateType_5(message):
    to_update_index = message.fields.index("AIS Version")
    ais_versions = ["ITU1371", "* future editions *", "* future editions *", "* future editions *"]
    message.dataParts[to_update_index] = ais_versions[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Ship Type")
    message.dataParts[to_update_index] = enums.codesForShipType[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Dimension to Bow")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Stern") 
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Port")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Starboard")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Position Fix Type")
    message.dataParts[to_update_index] = enums.ePFDFixTypes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("ETA Month (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA Day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA Minute (UTC)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Draught")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index]/10)

    to_update_index = message.fields.index("DTE")
    dte = ["Data terminal ready", "Data terminal not ready"]
    message.dataParts[to_update_index] = dte[message.dataParts[to_update_index]]

def updateType_6_1(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("ETA Month (UTC, Last)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA Day (UTC, Last)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA Hour (UTC, Last)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA Minute (UTC, Last)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA Month (UTC, Next)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA Day (UTC, Next)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA Hour (UTC, Next)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA Minute (UTC, Next)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Unit of Quantity")
    message.dataParts[to_update_index] = enums.cargoUnitCodes[message.dataParts[to_update_index]]

def updateType_6_2(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Month")
    if message.dataParts[to_update_index]  == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Day")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "N/A"

    concatenateFields(message.tidals, message.fields)
    concatenateDataparts(message.tidals, message.dataParts)

def updateType_6_3(message):
    to_update_index = message.fields.index("Number of Persons On Board")
    message.dataParts[to_update_index] = interpretPersonsOnBoard(message.dataParts[to_update_index])

def updateType_6_4(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Number of Persons On Board")
    message.dataParts[to_update_index] = interpretPersonsOnBoard(message.dataParts[to_update_index])

def updateType_6_5(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Month (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Minute (UTC)")
    if message.dataParts[to_update_index] == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Longitude")
    if message.dataParts[to_update_index] == 181:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Latitude")
    if message.dataParts[to_update_index] == 91:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])

def updateType_6_6(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Berth length")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "511 metres or more"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Berth Water Depth")
    if message.dataParts[to_update_index] == 25.5:
        message.dataParts[to_update_index] = "25.5 metres or more"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Mooring Position")
    message.dataParts[to_update_index] = enums.mooringPosition[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Month (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Minute (UTC)")
    if message.dataParts[to_update_index] == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Services Availability")
    services_availability = ["srvices unknown", "services known"]
    message.dataParts[to_update_index] = services_availability[message.dataParts[to_update_index]]

    to_update_first = message.fields.index("Agent")
    to_update_last = message.fields.index("Reserved for future 2")
    enum_values_numbers = message.dataParts[to_update_first:to_update_last+1]
    enum_ordered_values = [enums.serviceStatus[number] for number in enum_values_numbers]
    message.dataParts[to_update_first:to_update_last] = enum_ordered_values

    to_update_index = message.fields.index("Longitude")
    if message.dataParts[to_update_index] == 181:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Latitude")
    if message.dataParts[to_update_index] == 91:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])

def updateType_6_7(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Notice Description")
    message.dataParts[to_update_index] = enums.areaNoticeDescription[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Month (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Minute (UTC)")
    if message.dataParts[to_update_index] == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index["Duration"]
    if message.dataParts[to_update_index] == 262143:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "cancel this notice"
    else:
        message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])

    concatenateFields(message.shapes, message.fields)
    concatenateDataparts(message.shapes, message.dataParts)

def updateType_6_8(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Unit of Quantity")
    message.dataParts[to_update_index] = enums.cargoUnitCodes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Amount of Cargo")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "N/A"

    concatenateFields(message.cargos, message.fields)
    concatenateDataparts(message.cargos, message.dataParts)

def updateType_6_9(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Sender Class")
    sender_classes = [
        "ship", "authority", "reserved for future use", "reserved for future use", "reserved for future use",
        "reserved for future use", "reserved for future use", "reserved for future use"
    ]
    message.dataParts[to_update_index] = sender_classes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Route Type")
    message.dataParts[to_update_index] = enums.routeTypeCodes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Start Month (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Start Day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Start Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Start Minute (UTC)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Duration")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "cancel route"
    elif message.dataParts[to_update_index] == 262143:
        message.dataParts[to_update_index] == "N/A"
    else:
        message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])    

    concatenateFields(message.waypoints, message.fields)
    concatenateDataparts(message.waypoints, message.dataParts)

def updateType_6_10(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

def updateType_6_11(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Month")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Day")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"

    concatenateFields(message.tidals, message.fields)
    concatenateDataparts(message.tidals, message.dataParts)

def updateType_6_12(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("ETA month")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA day")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA hour")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("ETA minute")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Assisting Tugs")
    if message.dataParts[to_update_index] == 7:
        message.dataParts[to_update_index] = "unknown"

    to_update_index = message.fields.index("Air Draught")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "unknown"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index]*0.01)

def updateType_6_13(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("RTA month")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("RTA day")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("RTA hour")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("RTA minute")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Status")
    message.dataParts[to_update_index] = enums.statusCodes[message.dataParts[to_update_index]]

def updateType_6_14(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Number of Crew On Board")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "Unknown"

    to_update_index = message.fields.index("Number of Passengers On Board")
    if message.dataParts[to_update_index] == 8191:
        message.dataParts[to_update_index] = "Unknown"

    to_update_index = message.fields.index("Number of Personnel On Board")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "Unknown"

# Nie rozumiem co znaczy "units" dla pola z bitów 123-130
def updateType_6_15(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Analogue")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "Not Used"
    else:
        message.dataParts[to_update_index] = "{} V".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Analogue (ext. 1)")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "Not Used"
    else:
        message.dataParts[to_update_index] = "{} V".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Analogue (ext. 2)")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "Not Used"
    else:
        message.dataParts[to_update_index] = "{} V".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("RACON status")
    racon_statuses = ["no RACON installed", "RACON not monitored", "RACON operational", "RACON ERROR"]
    message.dataParts[to_update_index] = racon_statuses[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Light status")
    light_statuses = [" no light or no monitoring", "Light ON", "Light OFF", "Light ERROR"]
    message.dataparts[to_update_index] = light_statuses[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Health")
    health_statuses = ["Good Health", "Alarm"]
    message.dataParts[to_update_index] = health_statuses[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Position status")
    position_statuses = ["On position", "Off position"]
    message.dataParts[to_update_index] = position_statuses[message.dataParts[to_update_index]]

def updateType_7(message):
    concatenateFields(message.destinationMMSIs, message.fields)
    concatenateDataparts(message.destinationMMSIs, message.dataParts)

# Nie rozumiem co jest w "units" do Air Temperature i w Dew Point i w Water Temperature - możliwe błędy w interpretacji
def updateType_8_1(message):
    to_update_index = message.fields.index("Latitude")
    if message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} minutes (E)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index] = "{} minutes (W)".format(message.dataParts[to_update_index])
    else:
        message.dataParts[to_update_index] = "0 minutes"

    to_update_index = message.fields.index("Longitude")
    if message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} minutes (N)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index] = "{} minutes (S)".format(message.dataParts[to_update_index])
    else:
        message.dataParts[to_update_index] = "0 minutes"

    to_update_index = message.fields.index("Day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Minute (UTC)")
    if(message.dataParts[to_update_index]) == 63:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Average Wind Speed")
    if(message.dataParts[to_update_index] == 127):
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} knots (10-min avg)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Gust Speed")
    if(message.dataParts[to_update_index] == 127):
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} knots (10-min avg)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Wind Direction")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} degrees from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Wind Gust Direction")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} degrees from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Air Temperature")
    if message.dataParts[to_update_index] == 2047:
        message.dataPars[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "({} deg C)".format(message.dataParts[to_update_index]/10 - 60)

    to_update_index = message.fields.index("Relative Humidity")
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataparts[to_update_index] = "{}%".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Dew Point")
    if message.dataParts[to_update_index] == 1023:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "({} deg C)".format(message.dataParts[to_update_index]/10 - 20)

    to_update_index = message.fields.index("Air Pressure")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} hPA".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Pressure Tendency")
    tendencies = ["steady", "decreasing", "increasing", "N/A"]
    message.dataParts[to_update_index] = tendencies[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Horizontal Visibility")
    if message.dataparts[to_update_index] == 255:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} nautical miles".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Water Level")
    if message.dataparts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Water Level Trend")
    trends = ["steady", "decreasing", "increasing", "N/A"]
    message.dataParts[to_update_index] = trends[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Surface Current Speed")
    message.dataParts[to_update_index] = "{} knots".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Surface Current Direction")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Current Speed 2")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} knots".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Current Direction 2")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Measurement Depth 2")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Current Speed 3")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} knots".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Current Direction 3")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Measurement Depth 3")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Wave height")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Wave period")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} seconds".format(message.dataParts[to_update_index])
    
    to_update_index = message.fields.index("Wave direction")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Swell height")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Swell period")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} seconds".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Swell direction")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Sea state")
    message.dataParts[to_update_index] = enums.beaufortScale[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Water Temperature")
    if message.dataParts[to_update_index] == 1023:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg C".format(message.dataParts[to_update_index] - 10)

    to_update_index = message.fields.index("Precipitation")
    message.dataParts[to_update_index] = enums.precipitationType[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Salinity")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{}%".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Ice")
    ice = ["No", "Yes", "(reserved for future use)", "not avaliable"]
    message.dataParts[to_update_index] = ice[message.dataParts[to_update_index]]

def updateType_8_2(message):
    to_update_index = message.fields.index("Radius extension")
    if message.dataParts[to_update_index] == 10001:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Unit of extension")
    units = ["m", "km", "nm", "cables"]
    message.dataParts[to_update_index] = units[message[to_update_index]]

    to_update_index = message.fields.index("From day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("From month (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("From hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("From minute (UTC)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("To day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("To month (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("To hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("To minute (UTC)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

def updateType_8_3(message):
    to_update_index = message.fields.index("Air Draught")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 2047:
        message.dataParts[to_update_index] = "2047 or more metres"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index])

def updateType_8_4(message):
    concatenateFields(message.targets, message.fields)
    concatenateDataparts(message.targets, message.dataParts)

def updateType_8_5(message):
    to_update_index = message.fields.index("Longitude")
    if message.dataParts[to_update_index] == 181:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} minutes (E)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index] = "{} minutes (W)".format(message.dataParts[to_update_index])
    
    to_update_index = message.fields.index("Latitude")
    if message.dataParts[to_update_index] == 91:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} minutes (N)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index] = "{} minutes (S)".format(message.dataParts[to_update_index])   

    to_update_index = message.fields.index("Status of Signal")
    signal_statuses = ["N/A", "In regular service", "Irregular service", "(Reserved for future use)"]
    message.dataParts[to_update_index] = signal_statuses[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Signal In Service")
    message.dataParts[to_update_index] = enums.marineTrafficSignals[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Minute (UTC)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Expected Next Signal")
    message.dataParts[to_update_index] = enums.marineTrafficSignals[message.dataParts[to_update_index]]

# możliwa zła interpretacja "units" w Air Pressure; podobnie w Water Temperature
def updateType_8_6_1(message):
    to_update_index = message.fields.index("Longitude")
    if message.dataParts[to_update_index] == 181:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} minutes (E)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index] = "{} minutes (W)".format(message.dataParts[to_update_index])
    
    to_update_index = message.fields.index("Latitude")
    if message.dataParts[to_update_index] == 91:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} minutes (N)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index] = "{} minutes (S)".format(message.dataParts[to_update_index]) 

    to_update_index = message.fields.index("Day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Minute (UTC)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Visibility Limit")
    if message.dataParts[to_update_index] == 1:
        message.dataParts[to_update_index] = "Maximum range of the visibility equipment reached"

    to_update_index = message.fields.index("Horizontal Visibility")
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} nm".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Relative Humidity")
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{}%".format(message.dataParts[to_update_index])   

    to_update_index = message.fields.index("Average Wind Speed") 
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} knots (10-min avg)".format(message.dataParts[to_update_index])   

    to_update_index = message.fields.index("Wind Direction") 
    if message.dataParts[to_update_index] == 360:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Air Pressure") 
    if message.dataParts[to_update_index] == 403:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 402:
        message.dataParts[to_update_index] = "1201 or more hPa"
    else:
        message.dataParts[to_update_index] = "{} hPa".format(message.dataParts[to_update_index] + 400)  

    to_update_index = message.fields.index("Air Temperature")
    if message.dataParts[to_update_index] == -1024:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg C".format(message.dataParts[to_update_index])     

    to_update_index = message.fields.index("Water Temperature")
    if message.dataParts[to_update_index] == 601:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg C".format(message.dataParts[to_update_index] - 10)  

    to_update_index = message.fields.index("Wave period")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} seconds".format(message.dataParts[to_update_index])   

    to_update_index = message.fields.index("Wave height")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index]) 

    to_update_index = message.fields.index("Wave direction")
    if message.dataParts[to_update_index] == 360:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Swell height")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index]) 

    to_update_index = message.fields.index("Swell direction")
    if message.dataParts[to_update_index] == 360:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg from true north".format(message.dataParts[to_update_index])
        
    to_update_index = message.fields.index("Swell period")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} seconds".format(message.dataParts[to_update_index])   

# nie rozumiem co jest w lon i lat; w Height of cloud base - możliwa błędna interpretacja
def updateType_8_6_2(message):
    to_update_index = message.fields.index("Longitude")
    if message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} minutes (E)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index] = "{} minutes (W)".format(message.dataParts[to_update_index])
    
    to_update_index = message.fields.index("Latitude")
    if message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} minutes (N)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index] = "{} minutes (S)".format(message.dataParts[to_update_index])    

    to_update_index = message.fields.index("Month (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Minute (UTC)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Course Over Ground")
    message.dataParts[to_update_index] = "{} degrees (10-min avg)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Speed Over Ground")
    if message.dataParts[to_update_index] == 30:
        message.dataParts[to_update_index] = "15 m/s or more (10-min avg)"
    elif message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] == "N/A"
    else:
        message.dataParts[to_update_index] = "{} m/s (10-min avg0".format(message.dataParts[to_update_index]/2)

    to_update_index = message.fields.index("Heading of the ship")
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} degrees (10-min avg)".format(message.dataParts[to_update_index]*5)

    to_update_index = message.fields.index("Pressure at sea level")
    message.dataParts[to_update_index] = "{} hPa".format(message.dataParts[to_update_index] + 900)

    to_update_index = message.fields.index("Pressure Change")
    if message.dataParts[to_update_index] == 1023:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} hPa (3-hours avg)".format(message.dataParts[to_update_index] - 50)

    to_update_index = message.fields.index("Pressure Tendency")
    message.dataParts[to_update_index] = "{} (WMO BUFR table 010063; 3-hours avg)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("True Wind Direction")
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} degrees (10-min avg)".format(message.dataParts[to_update_index]*5)   

    to_update_index = message.fields.index("True Wind Speed")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m/s (10-min avg)".format(message.dataParts[to_update_index]*0.5) 

    to_update_index = message.fields.index("Relative Wind Direction")
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "calm"
    else:
        message.dataParts[to_update_index] = "{} degrees (10-min avg)".format(message.dataParts[to_update_index]*5)    

    to_update_index = message.fields.index("Relative Wind Speed")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m/s (10-min avg)".format(message.dataParts[to_update_index]*0.5) 
    
    to_update_index = message.fields.index("Maximum Gust Speed")
    if message.dataParts[to_update_index] == 255:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m/s (10-min avg)".format(message.dataParts[to_update_index]) 

    to_update_index = message.fields.index("Maximum Gust Direction")
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "calm"
    else:
        message.dataParts[to_update_index] = "{} degrees (10-min avg)".format(message.dataParts[to_update_index]*5)

    to_update_index = message.fields.index("Air Temperature")
    if message.dataParts[to_update_index] == 1023:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} K".format(message.dataParts[to_update_index] + 223) 

    to_update_index = message.fields.index("Relative Humidity")
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{}%".format(message.dataParts[to_update_index])   

    to_update_index = message.fields.index("Sea Surface Temperature")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} K".format(message.dataParts[to_update_index] + 268) 

    to_update_index = message.fields.index("Horizontal Visibility")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index]*message.dataParts[to_update_index]*13.0731)

    to_update_index = message.fields.index("Present Weather")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020003)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Past Weather 1")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020005)".format(message.dataParts[to_update_index]) 

    to_update_index = message.fields.index("Past Weather 1")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020004)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Total Cloud Cover")
    if message.dataParts[to_update_index] == 15 :
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{}%".format(message.dataParts[to_update_index]*10)   

    to_update_index = message.fields.index("Cloud amount (low)")
    if message.dataParts[to_update_index] == 15:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020011)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Cloud Type (low)")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020012)".format(message.dataParts[to_update_index])    

    to_update_index = message.fields.index("Cloud Type (middle)")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020012)".format(message.dataParts[to_update_index]) 

    to_update_index = message.fields.index("Cloud Type (high)")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020012)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Height of cloud base")
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 126:
        message.dataParts[to_update_index] = "more than 2500 m"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index]*2)

    to_update_index = message.fields.index("Period of Wind Waves")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} s".format(message.dataParts[to_update_index]) 

    to_update_index = message.fields.index("Height of Wind Waves")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index]*0.5) 

    to_update_index = message.fields.index("First Swell Direction")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "calm"
    else:
        message.dataParts[to_update_index] = "{} deg".format(message.dataParts[to_update_index]*10)

    to_update_index = message.fields.index("First Swell Period")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} s".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("First Swell Height")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Second Swell Direction")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "calm"
    else:
        message.dataParts[to_update_index] = "{} deg".format(message.dataParts[to_update_index]*10)

    to_update_index = message.fields.index("Second Swell Period")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} s".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Second Swell Height")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Ice deposit (thickness)")
    if message.dataParts[to_update_index] == 127:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} cm".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Rate of Ice Accretion")
    if message.dataParts[to_update_index] == 7:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020032)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Cause of Ice Accretion")
    if message.dataParts[to_update_index] == 7:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020033)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Sea Ice Concentration")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020034)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Amount and Type of Ice")
    if message.dataParts[to_update_index] == 15:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020035)".format(message.dataParts[to_update_index]) 

    to_update_index = message.fields.index("Ice Situation")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020036)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Ice Development")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} (BUFR table 020037)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Bearing of Ice Edge")
    if message.dataParts[to_update_index] == 15:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg".format(message.dataParts[to_update_index]*0.45) 

def updateType_8_7(message):
    to_update_index = message.fields.index("Notice Description")
    message.dataParts[to_update_index] = enums.areaNoticeDescription[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Month (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Day (UTC)")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Hour (UTC)")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Minute (UTC)")
    if message.dataParts[to_update_index] == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index["Duration"]
    if message.dataParts[to_update_index] == 262143:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "cancel this notice"
    else:
        message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])

    concatenateFields(message.shapes, message.fields)
    concatenateDataparts(message.shapes, message.dataParts)

def updateType_8_8(message):
    to_update_index = message.fields.index("Air Draught")
    if message.dataaParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 8191:
        message.dataParts[to_update_index] = "81.91 or more metres"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index]/100)

    to_update_first = message.fields.index("AIS Class A")
    to_update_last = message.fields.index("VDR/S-VDR")
    enum_values_numbers = message.dataParts[to_update_first:to_update_last+1]
    enum_ordered_values = [enums.sOLASStatus[number] for number in enum_values_numbers]
    message.dataParts[to_update_first:to_update_last] = enum_ordered_values  

    to_update_index = message.fields.index("Ice class")
    message.dataParrts[to_update_index] = enums.iceClass[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Shaft Horsepower")
    if message.dataParts[to_update_index] == 143:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 262142:
        message.dataParts[to_update_index] = "262142 hp or more"
    else:
        message.dataParts[to_update_index] = "{} hp".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("VHF Working Channel")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Gross Tonnage")
    if message.dataParts[to_update_index] == 143:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 262142:
        message.dataParts[to_update_index] = "262142 hp or more"
    else:
        message.dataParts[to_update_index] = "{} hp".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Laden or Ballast")
    lading_or_ballast = ["N/A", "Laden", "Ballast", "Not in use"]
    message.dataParts[to_update_index] = lading_or_ballast[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Heavy Fuel Oil Bunkered")
    heavys = ["N/A", "No", "Yes", "Not in use"]
    message.dataParts[to_update_index] = heavys[message.dataParts[to_update_index]]    

    to_update_index = message.fields.index("Light Fuel Oil Bunkered")
    lights = ["N/A", "No", "Yes", "Not in use"]
    message.dataParts[to_update_index] = lights[message.dataParts[to_update_index]] 

    to_update_index = message.fields.index("Diesel Oil Bunkered")
    diesels = ["N/A", "No", "Yes", "Not in use"]
    message.dataParts[to_update_index] = diesels[message.dataParts[to_update_index]] 

    to_update_first = message.fields.index("Total Bunker Oil")
    if message.dataParts[to_update_first] == 16381:
        message.dataParts[to_update_first] = "16381 tonnes or more"
    elif message.dataParts[to_update_first] == 16382:
        message.dataParts[to_update_first] = "N/A"
    else:
        message.dataParts[to_update_first] = "{} tonnes".format(message.dataParts[to_update_first])

    to_update_first = message.fields.index("Number of persons")
    if message.dataParts[to_update_first] == 0:
        message.dataPats[to_update_first] = "N/A"
    elif message.dataParts[to_update_first] == 8191:
        message.dataParts[to_update_first] = "8191 or more"

def updateType_8_9(message):
    concatenateFields(message.reports, message.fields)
    concatenateDataparts(message.reports, message.dataParts)

def updateType_8_10(message):
    to_update_index = message.fields.index("Sender Class")
    sender_classes = [
        "ship", "authority", "reserved for future use", "reserved for future use", "reserved for future use",
        "reserved for future use", "reserved for future use", "reserved for future use"
    ]
    message.dataParts[to_update_index] = sender_classes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Route Type")
    message.dataParts[to_update_index] = enums.routeTypeCodes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Start month")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Start day")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Start hour")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Start minute")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Duration")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "cancel route"
    elif message.dataParts[to_update_index] == 262143:
        message.dataParts[to_update_index] == "N/A"
    else:
        message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])    

    concatenateFields(message.waypoints, message.fields)
    concatenateDataparts(message.waypoints, message.dataParts)

def updateType_8_11(message):
    pass

def updateType_8_12(message):
    to_update_index = message.fields.index("Longitude")
    if message.dataParts[to_update_index] == 181:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} deg (E)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index] = "{} deg (W)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Longitude")
    if message.dataParts[to_update_index] == 91:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} deg (N)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index] = "{} deg (S)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Fix quality")
    message.dataParts[to_update_index] = interpretAccuracy(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Day")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Hour")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Minute")
    if(message.dataParts[to_update_index]) == 63:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Average Wind Speed")
    if(message.dataParts[to_update_index] == 127):
        message.dataParts[to_update_index] = "N/A"
    elif(message.dataParts[to_update_index] == 126):
        message.dataParts[to_update_index] = "126 or more knots (10-min avg)"
    else:
        message.dataParts[to_update_index] = "{} knots (10-min avg)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Gust Speed")
    if(message.dataParts[to_update_index] == 127):
        message.dataParts[to_update_index] = "N/A"
    elif(message.dataParts[to_update_index] == 126):
        message.dataParts[to_update_index] = "126 or more knots (10-min avg)"
    else:
        message.dataParts[to_update_index] = "{} knots (10-min avg)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Wind Direction")
    if message.dataParts[to_update_index] == 360:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} true bearing".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Wind Gust Direction")
    if message.dataParts[to_update_index] == 360:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} true bearing".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Air Temperature")
    if message.dataParts[to_update_index] == -102.4:
        message.dataPars[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg C".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Relative Humidity")
    if message.dataParts[to_update_index] == 101:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataparts[to_update_index] = "{}%".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Dew Point")
    if message.dataParts[to_update_index] == 50.1:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg C".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Air Pressure")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "799 hPa"
    elif message.dataParts[to_update_index] == 402:
        message.dataParts[to_update_index] = "1201 hPa or more"
    else:
        message.dataParts[to_update_index] = "{} hPA".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Pressure Tendency")
    tendencies = ["steady", "decreasing", "increasing", "N/A"]
    message.dataParts[to_update_index] = tendencies[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Maximum Visibility")
    achieved = ["Visibility as given below", "Visibilty greater than given below"]
    message.dataParts[to_update_index] = achieved[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Horizontal Visibility")
    if message.dataparts[to_update_index] == 12.7:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} nautical miles".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Water Level")
    if message.dataparts[to_update_index] == 40.01:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Water Level Trend")
    trends = ["steady", "decreasing", "increasing", "N/A"]
    message.dataParts[to_update_index] = trends[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Surface Current Speed")
    if message.dataParts[to_update_index] == 25.1:
        message.dataParts[to_update_index] = "25.1 knots or more"
    else:
        message.dataParts[to_update_index] = "{} knots".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Surface Current Direction")
    if message.dataParts[to_update_index] == 360:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Current Speed 2")
    if message.dataParts[to_update_index] == 25.5:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 25.1:
        message.dataParts[to_update_index] = "25.1 knots or more"
    else:
        message.dataParts[to_update_index] = "{} knots".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Current Direction 2")
    if message.dataParts[to_update_index] == 360:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg true bearing".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Measurement Depth 2")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Current Speed 3")
    if message.dataParts[to_update_index] == 25.5:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 25.1:
        message.dataParts[to_update_index] = "25.1 knots or more"
    else:
        message.dataParts[to_update_index] = "{} knots".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Current Direction 3")
    if message.dataParts[to_update_index] == 360:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg true bearing".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Measurement Depth 3")
    if message.dataParts[to_update_index] == 31:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Wave height")
    if message.dataParts[to_update_index] == 25.5:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 25.1:
        message.dataParts[to_update_index] = "25.1 metres or more"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Wave period")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} seconds".format(message.dataParts[to_update_index])
    
    to_update_index = message.fields.index("Wave direction")
    if message.dataParts[to_update_index] == 360:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg true bearing".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Swell height")
    if message.dataParts[to_update_index] == 25.5:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 25.1:
        message.dataParts[to_update_index] = "25.1 metres or more"
    else:
        message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Swell period")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} seconds".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Swell direction")
    if message.dataParts[to_update_index] == 360:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg from true north".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Sea state")
    message.dataParts[to_update_index] = enums.beaufortScale[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Water Temperature")
    if message.dataParts[to_update_index] == 50.1:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg C".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Precipitation")
    message.dataParts[to_update_index] = enums.precipitationType[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Salinity")
    if message.dataParts[to_update_index] == 51.1:
        message.dataParts[to_update_index] = "Sensor not available"
    elif message.dataParts[to_update_index] == 50.1:
        message.dataParts[to_update_index] = "50.1 percent or more"
    elif message.dataParts[to_update_index] == 50.1:
        message.dataParts[to_update_index] = "50.1 percent or more"
    else:
        message.dataParts[to_update_index] = "{}%".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Ice")
    ice = ["No", "Yes", "(reserved for future use)", "not avaliable"]
    message.dataParts[to_update_index] = ice[message.dataParts[to_update_index]]

def updateType_8_13(message):
    to_update_index = message.fields.index("Length of ship")
    message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index]/10)

    to_update_index = message.fields.index("Beam of ship")
    message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index]/10)

    to_update_index = message.fields.index("Ship/combination type")
    message.dataParts[to_update_index] = enums.eRIClassification[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Hazardous cargo")
    message.dataParts[to_update_index] = enums.hazardCode[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Draught")
    message.dataParts[to_update_index] = "{} m".format(message.dataParts[to_update_index]/100)

    to_update_index = message.fields.index("Loaded/Unloaded")
    message.dataParts[to_update_index] = enums.loadStatus[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Speed inf. quality")
    qualities = ["low/GNSS", "high"]
    message.dataParts[to_update_index] = qualities[message.dataParts[to_update_index]]
    to_update_index = message.fields.index("Course inf. quality")
    message.dataParts[to_update_index] = qualities[message.dataParts[to_update_index]]
    to_update_index = message.fields.index("Heading inf. quality")
    message.dataParts[to_update_index] = qualities[message.dataParts[to_update_index]]

def updateType_8_14(message):
    to_update_index = message.fields.index("Start Year")
    message.dataParts[to_update_index] = "{} year since 2000".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Start month")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Start day")
   
    to_update_index = message.fields.index("End Year")
    message.dataParts[to_update_index] = "{} year since 2000".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("End month")
    if(message.dataParts[to_update_index]) == 0:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("End day")

    to_update_index = message.fields.index("Start hour")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("Start minute")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("End hour")
    if(message.dataParts[to_update_index]) == 24:
        message.dataParts[to_update_index] = "N/A"
    to_update_index = message.fields.index("End minute")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("Start Longitude")
    message.dataParts[to_update_index] = calculateLongitude(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Start Latitude")
    message.dataParts[to_update_index] = calculateLatitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("End Longitude")
    message.dataParts[to_update_index] = calculateLongitude(message.dataParts[to_update_index])
    to_update_index = message.fields.index("End Latitude")
    message.dataParts[to_update_index] = calculateLatitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Type")
    message.dataParts[to_update_index] = enums.eMMATypeCodes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Classification")
    classes = ["Unknown", "Slight", "Medium", "Strong"]
    message.dataParts[to_update_index] = classes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Wind Direction")
    message.dataParts[to_update_index] = enums.eMMAWinds[message.dataParts[to_update_index]]

def updateType_8_15(message):
    concatenateFields(message.gauges, message.fields)
    concatenateDataparts(message.gauges, message.dataParts)

def updateType_8_16(message):
    to_update_index = message.fields.index("Signal Longitude")
    message.dataParts[to_update_index] = calculateLongitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Signal Latitude")
    message.dataParts[to_update_index] = calculateLatitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Signal form")
    message.dataParts[to_update_index] = "{} (as in International Standard for tracking and tracing on Inland Waterways, Annex C)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Signal orientation")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} deg".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Direction of impact")
    message.dataParts[to_update_index] = enums.signalImpact[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Light Status")
    message.dataParts[to_update_index] = enums.signalStatus[message.dataParts[to_update_index]]

def updateType_9(message):
    to_update_index = message.fields.index("Altitude")
    if message.dataParts[to_update_index] == 4095:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 4094:
        message.dataParts[to_update_index] = "4094 metres or more"
    else:
        message.dataParts[to_update_index] = "{} metres".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("SOG")
    if message.dataParts[to_update_index] == 1023:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] == 1022:
        message.dataParts[to_update_index] = "1022 knots or more".format(message.dataParts[to_update_index])
    else:
        message.dataParts[to_update_index] = "{} knots".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Position Accuracy")
    message.dataParts[to_update_index] = interpretAccuracy(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Longitude")
    message.dataParts[to_update_index] = calculateLongitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Latitude")
    message.dataParts[to_update_index] = calculateLatitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Course Over Ground")
    if message.dataParts[to_update_index] == 360.0:
        message.dataParts[to_update_index] = "not available"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + " degrees"

    to_update_index = message.fields.index("Time Stamp")
    if message.dataParts[to_update_index] == 60:
        message.dataParts[to_update_index] = "not available"
    elif message.dataParts[to_update_index] == 61:
        message.dataParts[to_update_index] = "positioning system in manual input mode"
    elif message.dataParts[to_update_index] == 62:
        message.dataParts[to_update_index] == "Electronic Position Fixing System operates in estimated (dead reckoning) mode"
    elif message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] == "positioning system is inoperative"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + "second of UTC timestamp"

    to_update_index = message.fields.index("DTE")
    dtes = ["Data terminal ready", "Data terminal not ready"]
    message.dataParts[to_update_index] = dtes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("RAIM flag")
    message.dataParts[to_update_index] = interpretRAIMflag(message.dataParts[to_update_index])

def updateType_10(message):
    pass

def updateType_11(message):
    updateType_4(message)

def updateType_12(message):
    to_update_index = message.fields.index("Retransmit flag")
    message.dataParts[to_update_index] = interpretRetransmitFlag(message.dataParts[to_update_index])

def updateType_13(message):
    updateType_7(message)

def updateType_14(message):
    pass

def updateType_15(message):
    pass

def updateType_16(message):
    #IALA - błąd 404; podawana wartość liczbowa tak oznaczonych pól
    pass

def updateType_17(message):
    to_update_index = message.fields.index("Longitude")
    message.dataParts[to_update_index] = calculateLongitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Latitude")
    message.dataParts[to_update_index] = calculateLatitude(message.dataParts[to_update_index])

def updateType_18(message):
    to_update_index = message.fields.index("Speed Over Ground")
    message.dataParts[to_update_index] = calculateSpeedOverGround(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Position Accuracy")
    message.dataParts[to_update_index] = interpretAccuracy(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Longitude")
    message.dataParts[to_update_index] = calculateLongitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Latitude")
    message.dataParts[to_update_index] = calculateLatitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("True Heading")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "not available"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + " degrees"

    to_update_index = message.fields.index("Time Stamp")
    if message.dataParts[to_update_index] == 60:
        message.dataParts[to_update_index] = "not available"
    elif message.dataParts[to_update_index] == 61:
        message.dataParts[to_update_index] = "positioning system in manual input mode"
    elif message.dataParts[to_update_index] == 62:
        message.dataParts[to_update_index] == "Electronic Position Fixing System operates in estimated (dead reckoning) mode"
    elif message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] == "positioning system is inoperative"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + "second of UTC timestamp"

    to_update_index = message.fields.index("CS Unit")
    cs_unit = ["Class B SOTDMA unit", "Class B CS (Carrier Sense) unit"]
    message.dataParts[to_update_index] = cs_unit[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Display flag")
    display_flag = ["No visual display", "Has display"]
    message.dataParts[to_update_index] = display_flag[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("DSC Flag")
    dsc = ["unit NOT attached to a VHF voice radio with DSC capability", "unit attached to a VHF voice radio with DSC capability"]
    message.dataParts[to_update_index] = dsc[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Band Flag")
    dsc = ["unit can NOT use any part of the marine channel", "unit can use any part of the marine channel"]
    message.dataParts[to_update_index] = dsc[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Message 22 flag")
    dsc = ["unit can NOT accept a channel assignment via Message Type 22", "unit can accept a channel assignment via Message Type 22"]
    message.dataParts[to_update_index] = dsc[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Assigned")
    assigned = ["Autonomous mode (default)", "Assigned mode"]
    message.dataParts[to_update_index] = assigned[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("RAIM flag")
    message.dataParts[to_update_index] = interpretRAIMflag(message.dataParts[to_update_index])

def updateType_19(message):
    to_update_index = message.fields.index("Speed Over Ground")
    message.dataParts[to_update_index] = calculateSpeedOverGround(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Position Accuracy")
    message.dataParts[to_update_index] = interpretAccuracy(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Longitude")
    message.dataParts[to_update_index] = calculateLongitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Latitude")
    message.dataParts[to_update_index] = calculateLatitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Course Over Ground")
    if message.dataParts[to_update_index] == 360.0:
        message.dataParts[to_update_index] = "not available"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + " degrees"

    to_update_index = message.fields.index("True Heading")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "not available"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + " degrees"

    to_update_index = message.fields.index("Time Stamp")
    if message.dataParts[to_update_index] == 60:
        message.dataParts[to_update_index] = "not available"
    elif message.dataParts[to_update_index] == 61:
        message.dataParts[to_update_index] = "positioning system in manual input mode"
    elif message.dataParts[to_update_index] == 62:
        message.dataParts[to_update_index] == "Electronic Position Fixing System operates in estimated (dead reckoning) mode"
    elif message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] == "positioning system is inoperative"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + "second of UTC timestamp"

    to_update_index = message.fields.index("Type of ship and cargo")
    message.dataParts[to_update_index] = enums.codesForShipType[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Dimension to Bow")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Stern") 
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Port")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Starboard")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Position Fix Type")
    message.dataParts[to_update_index] = enums.ePFDFixTypes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("RAIM flag")
    message.dataParts[to_update_index] = interpretRAIMflag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("DTE")
    dte = ["Data terminal ready", "Data terminal not ready"]
    message.dataParts[to_update_index] = dte[message.dataParts[to_update_index]]

def updateType_20(message):
    concatenateFields(message.reservedSlots, message.fields)
    concatenateDataparts(message.reservedSlots, message.dataParts)

def updateType_21(message):
    to_update_index = message.fields.index("Aid type")
    message.dataParts[to_update_index] = enums.navaidTypes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Position Accuracy")
    message.dataParts[to_update_index] = interpretAccuracy(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Longitude")
    message.dataParts[to_update_index] = calculateLongitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Latitude")
    message.dataParts[to_update_index] = calculateLatitude(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Dimension to Bow")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Stern") 
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Port")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Starboard")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Type of EPFD")
    message.dataParts[to_update_index] = enums.ePFDFixTypes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Second (UTC)")
    if(message.dataParts[to_update_index]) == 60:
        message.dataParts[to_update_index] = "N/A"

    to_update_index = message.fields.index("RAIM flag")
    message.dataParts[to_update_index] = interpretRAIMflag(message.dataParts[to_update_index])

def updateType_22(message):
    to_update_index = message.fields.index("Power")
    power = ["Low", "High"]
    message.dataParts[to_update_index] = power[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("NE Longitude")
    message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("NE Latitude")
    message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("SW Longitude")
    message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("SW Latitude")
    message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Addressed")
    addressed = ["Broadcast", "Addressed"]
    message.dataParts[to_update_index] = addressed[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Channel A Band")
    channel_a_band = ["Default", "12,5 kHz"]
    message.dataParts[to_update_index] = channel_a_band[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Channel B Band")
    channel_b_band = ["Default", "12,5 kHz"]
    message.dataParts[to_update_index] = channel_b_band[message.dataParts[to_update_index]]

def updateType_23(message):
    to_update_index = message.fields.index("NE Longitude")
    message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("NE Latitude")
    message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("SW Longitude")
    message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("SW Latitude")
    message.dataParts[to_update_index] = "{} minutes".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Station Type")
    message.dataParts[to_update_index] = enums.stationTypes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Ship Type")
    message.dataParts[to_update_index] = enums.codesForShipType[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Tx/Rx Mode")
    transmit_modes = ["TxA/TxB, RxA/RxB (default)", "TxA, RxA/RxB", "TxB, RxA/RxB", "Reserved for Future Use"]
    message.dataParts[to_update_index] = transmit_modes[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Report Interval")
    message.dataParts[to_update_index] = enums.stationIntervals[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Quiet Time")
    if message.dataParts[to_update_index] == 0:
        message.dataParts[to_update_index] = "none"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + " minutes"

def updateType_24_A(message):
    pass

def updateType_24_B(message):
    to_update_index = message.fields.index("Ship Type")
    message.dataParts[to_update_index] = enums.codesForShipType[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Dimension to Bow")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Stern") 
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Port")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])
    to_update_index = message.fields.index("Dimension to Starboard")
    message.dataParts[to_update_index] = "{} meters".format(message.dataParts[to_update_index])

def updateType_25(message):
    to_update_index = message.fields.index("Destination indicator")
    destination_indicator = ["Broadcast", "Addressed"]
    message.dataParts[to_update_index] = destination_indicator[message.dataParts[to_update_index]]

def updateType_26(message):
    to_update_index = message.fields.index("Destination indicator")
    destination_indicator = ["Broadcast", "Addressed"]
    message.dataParts[to_update_index] = destination_indicator[message.dataParts[to_update_index]]

def updateType_27(message):
    to_update_index = message.fields.index("Position Accuracy")
    message.dataParts[to_update_index] = interpretAccuracy(message.dataParts[to_update_index])

    to_update_index = message.fields.index("RAIM flag")
    message.dataParts[to_update_index] = interpretRAIMflag(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Navigation Status")
    message.dataParts[to_update_index] = enums.navigationStatus[message.dataParts[to_update_index]]

    to_update_index = message.fields.index("Longitude")
    if message.dataParts[to_update_index] == 181:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} (East)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index]  = "{} (West)".format(message.dataParts[to_update_index])

    to_update_index = message.fields.index("Latitude")
    if message.dataParts[to_update_index] == 91:
        message.dataParts[to_update_index] = "N/A"
    elif message.dataParts[to_update_index] > 0:
        message.dataParts[to_update_index] = "{} (North)".format(message.dataParts[to_update_index])
    elif message.dataParts[to_update_index] < 0:
        message.dataParts[to_update_index]  = "{} (South)".format(message.dataParts[to_update_index])    

    to_update_index = message.fields.index("Speed Over Ground")
    if message.dataParts[to_update_index] == 63:
        message.dataParts[to_update_index] = "N/A"
    else:
        message.dataParts[to_update_index] = "{} knots".format(message.dataParts[to_update_index])

    # Chyba coś pomieszali w opisie, że jest 511 not available przy tym
    #   Odp: być może, ale nie nam to osądzać; w niektórych typach wyżej było 360 dla N/A a czasem 511 (dla takiego samego
    #   parametru, niejednokrotnie) - różnica może wynikać z zalecenia, na którego podstawie był robiony dany opis
    to_update_index = message.fields.index("Course Over Ground")
    if message.dataParts[to_update_index] == 511:
        message.dataParts[to_update_index] = "not available"
    else:
        message.dataParts[to_update_index] = str(message.dataParts[to_update_index]) + " degrees"

    to_update_index = message.fields.index("GNSS Position status")
    gnss_position_status = ["Current GNSS position", "Not GNSS position (default)"]
    message.dataParts[to_update_index] = gnss_position_status[message.dataParts[to_update_index]]






