from . import counters
def read_message(file):   
    message_lines = read_message_lines(file)
    counters.message_counter += 1
    
    # Nie przeczytano żadnej linii - koniec pliku
    if not message_lines:
        return "eof"

    lines_payloads = []
    all_checksums_ok = True
    for line in message_lines:
        line_data = extract_data(line)
        received_checksum = extract_checksum(line)
        calculated_checksum = calculate_checksum(line_data)
        checksum_ok = compare_checksums(received_checksum, calculated_checksum)
        if checksum_ok:
            payload = extract_payload(line)
            line_bin_payload = convert_to_binary(payload)
            lines_payloads.append(line_bin_payload)
        else:
            all_checksums_ok = False
            break
            
    if all_checksums_ok:
        message_bin_payload = concatenate_payloads(lines_payloads)
        message = add_type(message_bin_payload)
        return message
    else:
        return "message data error"

def read_message_lines(file):
    lines = []
    while True:
        line = file.readline()       
        # koniec pliku
        if len(line) == 0:
            break
        
        # Koniec, jeśli przeczytano wszystkie linie wiadomości
        if line[27] == line[29]:
            lines.append(line)
            break
        else:
            lines.append(line)

    if len(lines) == 1:
        counters.single_line_counter += 1
    elif len(lines) == 2:
        counters.double_line_counter +=1

    return lines

def extract_data(line):
    data_start = line.find("!") + 1
    data_stop = line.find("*", data_start)
    return line[data_start:data_stop]   

def extract_checksum(line):
    data_checksum_start = line.find("*") + 1
    data_checksum_stop = data_checksum_start + 2
    return line[data_checksum_start:data_checksum_stop]

def calculate_checksum(line_data):
    binary_format_data = ''.join(['{0:08b}'.format(ord(x)) for x in line_data])
    i = 0
    xor = 0
    j = 8
    while i < len(binary_format_data):
        xor = xor ^ int(binary_format_data[i:j], 2)
        i += 8
        j = 8 + i
    return xor

def compare_checksums(received, calculated):
    return hex(calculated) == hex(int(received, 16))

def extract_payload(data):
    data_payload_start = data.find(",")
    data_payload_start = data.find(",", data_payload_start + 7)
    data_payload_stop = data.find("*") - 2
    return data[data_payload_start + 1: data_payload_stop]

def convert_to_binary(payload):
    letters_payloads = []
    w = 0    
    while w < len(payload):
        bin_letter = convert_letter_to_bin(payload[w])
        letters_payloads.append(bin_letter)
        w += 1
    concatenated_payload = concatenate_payloads(letters_payloads)
    return concatenated_payload

def convert_letter_to_bin(letter):
    temp = ord(letter) - 48
    if temp > 40:
        temp -=8
    return bin(temp)[2:].zfill(6)

def concatenate_payloads(payloads):
    return ''.join(payloads)

def add_type(payload):

    if is_empty(payload):
        counters.empty_payload_counter += 1
        return "message data error"
    else:
        type_bin = payload[0:6]
        type_dec = int(type_bin, 2)

        message = {
            "type": str(type_dec),
            "message": payload
        }
        
        counters.message_type_counters[type_dec-1] += 1

        if type_dec == 6:
            dac = int(payload[72:82], 2)
            fid = int(payload[82:88], 2)
            if dac == 1 and fid == 12:
                message['type'] = '6_1'
            elif dac == 1 and fid == 14:
                message['type'] = '6_2'
            elif dac == 1 and fid == 16:
                message['type'] = '6_3'
            elif dac == 1 and fid == 18:
                message['type'] = '6_4'
            elif dac == 1 and fid == 20:
                message['type'] = '6_5'
            elif dac == 1 and fid == 23:
                message['type'] = '6_6'
            elif dac == 1 and fid == 25:
                message['type'] = '6_7'
            elif dac == 1 and fid == 28:
                message['type'] = '6_8'
            elif dac == 1 and fid == 30:
                message['type'] = '6_9'
            elif dac == 1 and fid == 32:
                message['type'] = '6_10'
            elif dac == 200 and fid == 21:
                message['type'] = '6_11'
            elif dac == 200 and fid == 22:
                message['type'] = '6_12'
            elif dac == 200 and fid == 55:
                message['type'] = '6_13'
            elif dac == 235 and fid == 10:
                message['type'] = '6_14'
            elif dac == 250 and fid == 10:
                message['type'] = '6_15'

        elif type_dec == 8:
            dac = int(payload[40:50], 2)
            fid = int(payload[50:56], 2)
            if dac == 1 and fid == 11:
                message['type'] = '8_1'
            elif dac == 1 and fid == 13:
                message['type'] = '8_2'
            elif dac == 1 and fid == 15:
                message['type'] = '8_3'
            elif dac == 1 and fid == 17:
                message['type'] = '8_4'
            elif dac == 1 and fid == 19:
                message['type'] = '8_5'
            elif dac == 1 and fid == 21:
                message['type'] = '8_6'
            elif dac == 1 and fid == 22:
                message['type'] = '8_7'
            elif dac == 1 and fid == 24:
                message['type'] = '8_8'
            elif dac == 1 and fid == 26:
                message['type'] = '8_9'
            elif dac == 1 and fid == 27:
                message['type'] = '8_10'
            elif dac == 1 and fid == 29:
                message['type'] = '8_11'
            elif dac == 1 and fid == 31:
                message['type'] = '8_12'
            elif dac == 200 and fid == 10:
                message['type'] = '8_13'
            elif dac == 200 and fid == 23:
                message['type'] = '8_14'
            elif dac == 200 and fid == 24:
                message['type'] = '8_15'
            elif dac == 200 and fid == 40:
                message['type'] = '8_16'
        return message

def is_empty(payload):
    return len(payload) == 0

def get_stats():
    return {"all":counters.message_counter,
            "types":counters.message_type_counters,
            "single":counters.single_line_counter,
            "double":counters.double_line_counter,
            "defective":counters.defective_message_counter,
            "empty":counters.empty_payload_counter
            }